#!/usr/bin/env python3
import ctypes
from ctypes import *
import syslog
from enum import Enum
import psutil
import os


fun_ppro = 5
class kysec_message_type(Enum):
    PPRO_ADD = 9
    PPRO_DEL = 10
    PPRO_PID = 12

class kysec_ppro_type(Enum):
    PPRO_SUBS = 1
    PPRO_KEY = 2

class kysec_func_status(Enum):
	FUNC_DISABLED = 0
	FUNC_ENABLED = 1

class kysec_ppro_kernel(Structure):
    _fields_ = [
        ('path', ctypes.c_char * 4096),
        ('hash', ctypes.c_char * 65),
        ('type', ctypes.c_int)
    ]

class KeyProcessControl:
    def __init__(self):
        self.extend_so = CDLL("libkysec_extend.so.0")

    def get_pids_by_path(self, path):
        pids = []
        for proc in psutil.process_iter(['pid', 'exe']):
            try:
                if proc.info['exe'] and os.path.samefile(proc.info['exe'], path):
                    pids.append(proc.info['pid'])
            except (psutil.NoSuchProcess, FileNotFoundError, PermissionError):
                continue
        return pids

    def kysec_process_enable(self):
        ret = self.extend_so.kysec_getstatus()
        if ret != 2:
            syslog.syslog(syslog.LOG_DEBUG, "kysec get status error")
            return -1

        ret = self.extend_so.kysec_get_func_status(c_int(fun_ppro))
        if ret == 1 or ret == 2:
            return 0

        if ret == 0:
            ret = self.extend_so.kysec_set_func_status(c_int(fun_ppro), c_int(kysec_func_status.FUNC_ENABLED.value))
            if ret != 0:
                syslog.syslog(syslog.LOG_DEBUG, "kysec set func status error")
                return -1

            ret = self.extend_so.kysec_conf_set("kysec_ppro", c_int(kysec_func_status.FUNC_ENABLED.value))
            if ret != 0:
                syslog.syslog(syslog.LOG_DEBUG, "kysec set conf error")
                return -1

        return 0

    def kysec_ppro_add(self, path):
        if not path:
            syslog.syslog(syslog.LOG_DEBUG, "path is invalid")
            return -1

        self.extend_so.kysec_ppro_read_node_with_path.restype = POINTER(kysec_ppro_kernel)
        ppro = self.extend_so.kysec_ppro_read_node_with_path(c_char_p(bytes(path, encoding = "utf8")))
        if ppro:
            ret = self.extend_so.kysec_ppro_update(c_char_p(bytes(path, encoding = "utf8")), c_int(kysec_ppro_type.PPRO_KEY.value))
        else:
            ret = self.extend_so.kysec_ppro_add(c_char_p(bytes(path, encoding = "utf8")), c_int(kysec_ppro_type.PPRO_KEY.value))
        if ret != 0:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro add error, path = {}, ret = {}".format(path, ret))
            return -1

        self.extend_so.kysec_ppro_get_path_hash.restype = c_char_p
        hash = self.extend_so.kysec_ppro_get_path_hash(c_char_p(bytes(path, encoding = "utf8")))
        if not hash:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro get hash error, path = {}".format(path))
            return -1

        ret = self.extend_so.kysec_ppro_communicate_with_kernel(c_int(kysec_message_type.PPRO_ADD.value), hash, kysec_ppro_type.PPRO_KEY.value)
        syslog.syslog(syslog.LOG_DEBUG, "add type = {}, path = {}, hash = {}, ret = {}".format(kysec_ppro_type.PPRO_KEY.value, path, str(hash, "utf8"), ret))
        if ret != 0:
            syslog.syslog(syslog.LOG_DEBUG, "kysec add ppro communicate with kernel error, path = {}, ret = {}".format(path, ret))
            return -1

        pids = self.get_pids_by_path(path)
        for pid in pids:
            if pid > 0:
                ret = self.extend_so.kysec_ppro_communicate_with_kernel(c_int(kysec_message_type.PPRO_PID.value), c_char_p(bytes("add", encoding = "utf8")), c_int(pid))
                if ret != 0:
                    syslog.syslog(syslog.LOG_DEBUG, "send ppro data to kernel failed, path = {}, pid = {}, ret = {}".format(path, pid, ret))
                    return -1
    
        return 0

    def kysec_ppro_delete(self, path):
        if not path:
            syslog.syslog(syslog.LOG_DEBUG, "path is invalid")
            return -1
        
        self.extend_so.kysec_ppro_read_node_with_path.restype = POINTER(kysec_ppro_kernel)
        ppro = self.extend_so.kysec_ppro_read_node_with_path(c_char_p(bytes(path, encoding = "utf8")))
        if not ppro:
            return 0
        
        ret = self.extend_so.kysec_ppro_remove(c_char_p(bytes(path, encoding = "utf8")))
        if ret != 0:
            syslog.syslog(syslog.LOG_DEBUG, "kysec ppro remove error, path = {}, ret = {}".format(path, ret))
            return -1
        
        ret = self.extend_so.kysec_ppro_communicate_with_kernel(c_int(kysec_message_type.PPRO_DEL.value), ppro.contents.hash, ppro.contents.type)
        syslog.syslog(syslog.LOG_DEBUG, "del type = {}, path = {}, hash = {}, ret = {}".format(ppro.contents.type, str(ppro.contents.path, "utf8"), str(ppro.contents.hash, "utf8"), ret))
        if ret != 0:
            syslog.syslog(syslog.LOG_DEBUG, "kysec remove ppro communicate with kernel error, path = {}, ret = {}".format(path, ret))
            return -1

        pids = self.get_pids_by_path(path)
        for pid in pids:
            if pid > 0:
                ret = self.extend_so.kysec_ppro_communicate_with_kernel(c_int(kysec_message_type.PPRO_PID.value), c_char_p(bytes("del", encoding = "utf8")), c_int(pid))
                if ret != 0:
                    syslog.syslog(syslog.LOG_DEBUG, "send ppro data to kernel failed, path = {}, pid = {}, ret = {}".format(path, pid, ret))
                    return -1
        return 0

    def kysec_ppro_get(self):
        self.extend_so.kysec_ppro_read_all_data.restype = POINTER(kysec_ppro_kernel)
        count = c_int(0)
        list = self.extend_so.kysec_ppro_read_all_data(byref(count))
        return list, count.value

    def kysec_ppro_clear(self):
        list, count = self.kysec_ppro_get()
        for i in range(0, count):
            # print(list[i].type, list[i].path, list[i].hash)
            if list[i].type == kysec_ppro_type.PPRO_KEY.value:
                ret = self.kysec_ppro_delete(str(list[i].path, "utf8"))
                if ret != 0:
                    syslog.syslog(syslog.LOG_DEBUG, "kysec ppro delete error")
                    return -1
        return 0
